/*
 * StudentRegister.java
 *
 * version: 30th November, 14:21
 * authors: yanas
 */
package cz.yanas.pb162.students;

import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.SortedSet;
import java.util.TreeSet;

/**
 * Represents a student register.
 *
 * @author yanas
 */
public final class StudentRegister {

    /** student's identification numbers generator */
    private static int idsGenerator = 1;

    /** a map associating student's identification numbers with students */
    private final Map<Integer, Student> students = new HashMap<Integer, Student>();

    /**
     * Creates an instance of the Student class using the given parameters and an unique
     * student's identification number.
     * 
     * @param surname student's surname
     * @param name student's name
     * @param dateOfBirth student's date of birth
     *
     * @return an instance of the Student class with unique id
     *
     * @throws IllegalArgumentException if the id is invalid
     * @throws NullPointerException if any argument is null
     */
    public Student createStudent(String surname, String name, Date dateOfBirth) {
        // throw a NullPointerException if the surname is null
        if (surname == null) {
            throw new NullPointerException("surname");
        }

        // throw a NullPointerException if the name is null
        if (name == null) {
            throw new NullPointerException("name");
        }

        // throw a NullPointerException if the dateOfBirth is null
        if (dateOfBirth == null) {
            throw new NullPointerException("dateOfBirth");
        }

        // get an unique student's identification number and increment the generator
        int uniqueId = idsGenerator++;

        // create a new student with the given parameters
        Student student = new Student(uniqueId, surname, name, dateOfBirth);
        students.put(uniqueId, student);

        // finally return the student
        return student;
    }

    /**
     * Returns true if the register contains a student with the given id, false otherwise.
     *
     * @param id an id of the student to be checked
     *
     * @return true if the register contains a student with the given id
     */
    public boolean containsStudent(int id) {
        return students.containsKey(id);
    }

    /**
     * Returns the student with the given id (if the student is contained in the register).
     *
     * @param id the id of the student to be returned
     *
     * @return the student with the given id
     *
     * @throws IllegalArgumentException if the id is invalid
     */
    public Student getStudentById(int id) {
        // throw an IllegalArgumentException if the id is invalid
        if (!containsStudent(id)) {
            throw new IllegalArgumentException("id");
        }

        // simply return the student associated with the given id
        return students.get(id);
    }

    /**
     * Returns a set of students sorted by their full name in ascending order.
     *
     * @return a set of students sorted by their full name in ascending order
     */
    public SortedSet<Student> getStudentsSortedByFullName() {
        // create a sorted set of students with the corresponding comparator
        SortedSet<Student> sortedStudents =
                new TreeSet<Student>(StudentFullNameComparator.getInstance());
        sortedStudents.addAll(students.values());

        return sortedStudents;
    }

    /**
     * Returns a set of students sorted by their date of birth in descending order.
     *
     * @return a set of students sorted by their date of birth in descending order
     */
    public SortedSet<Student> getStudentsSortedByDateOfBirth() {
        // create a sorted set of students with the corresponding comparator
        SortedSet<Student> sortedStudents = new TreeSet<Student>(
                Collections.reverseOrder(StudentDateOfBirthComparator.getInstance()));
        sortedStudents.addAll(students.values());

        return sortedStudents;
    }
}
